/*
--             This file is part of the New World OS project
--                    Copyright (C) 2008 QRW Software
--           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--                      http://www.qrwsoftware.com
--                      http://nwos.sourceforge.com
--
--   This program is free software: you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation, either version 3 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program, in the file LICENSE.  If not, see 
--   <http://www.gnu.org/licenses/>.
--
--   You can also contact me via paper mail at:
--
--      QRW Software
--      P.O. Box 27511
--      Salt Lake City, UT 84127-0511, USA.
--
--
-- $Log: dump_ids.c,v $
-- Revision 1.2  2008/04/28 11:16:19  jsedwards
-- Fixed include of objectify.h.
--
-- Revision 1.1  2008/04/28 11:07:32  jsedwards
-- Initial version created on 2008/01/01.
--
*/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>    /* define sleep() */

#include "../objectify_private.h"


uint32 convert_4_uint8_to_uint32(uint8 byte[4])
{
  return ((uint32)byte[0] << 24) | ((uint32)byte[1] << 16) | ((uint32)byte[2] << 8) | (uint32)byte[3];
}


int main(int argc, char* argv[])
{
    FILE* fp1;
    unsigned char buf1a[FILE_BLOCK_SIZE];
    int block1;
    size_t read1;


    if (argc != 2)
    {
	fprintf(stderr, "usage: %s file\n", argv[0]);
	exit(1);
    }


    /* Open the file and check it */

    fp1 = fopen(argv[1], "r");
    if (fp1 == NULL)
    {
	perror(argv[1]);
	exit(1);
    }

    read1 = fread(buf1a, 1, sizeof(buf1a), fp1);

    if (read1 != FILE_BLOCK_SIZE)
    {
	if (ferror(fp1))
	{
	    perror(argv[1]);
	}
	else
	{
	    fprintf(stderr, "Unexpected end of file: %s\n", argv[1]);
	}
	fclose(fp1);
	exit(1);
    }


    read1 = fread(buf1a, 1, sizeof(buf1a), fp1);

    if (read1 != FILE_BLOCK_SIZE)
    {
	if (ferror(fp1))
	{
	    perror(argv[1]);
	}
	else
	{
	    fprintf(stderr, "Unexpected end of file: %s\n", argv[1]);
	}
	fclose(fp1);
	exit(1);
    }

    block1 = 0;

    read1 = fread(buf1a, 1, sizeof(buf1a), fp1);

    while (!feof(fp1) && read1 == FILE_BLOCK_SIZE)
    {
	if (buf1a[0] != 0 || buf1a[1] != 0 || buf1a[2] != 0 || buf1a[3] != 0)
	{
	    printf("\n%s block %d - first four bytes not zero: %02x%02x%02x%02x\n",
		   argv[1], block1, buf1a[0], buf1a[1], buf1a[2], buf1a[3]);
	    break;
	}

	printf("%02x%02x%02x%02x: changed\n", buf1a[4], buf1a[5], buf1a[6], buf1a[7]);

	read1 = fread(buf1a, 1, sizeof(buf1a), fp1);
	block1++;
    }

    printf("blocks: %d\n", block1);

    if (!feof(fp1) && read1 != FILE_BLOCK_SIZE)
    {
	perror(argv[1]);
    }

    fclose(fp1);

    return 0;
}
